function [filterBank obsPointsBinary negPointsBinary lattice] = prepareDataForAdaboostTest(subsampleStep,posImageNames,negImageNames,posFolder,negFolder,threshold,selectedFeatures)
% (Much of code here is re-use of ABlearn.m .)
% lattice: subsampled positions, a 2D matrix
% obsPoints: original gabor responses
% The only difference from 'prepareDataForAdaboost.m' is that <threshold> is given.

% Get Gabor responses for pos and neg images on selected basis.
scale = .7;  % scale of Gabors
norient = 15;  % number of orientations
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
allfilter = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor

% =====================================================
% Positive images
% =====================================================
Iname = posImageNames;
nimage = size(Iname, 1);
Isize = zeros(nimage, 2); 
for i = 1 : nimage
    tmpIm = imread([posFolder '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm), 1); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the testing images to be the same

% filter the pos images
allfilteredPos = applyfilterfftsame(I, allfilter);  % filter testing images
mex Cmax.c; 
Cmax(nimage, norient, allfilteredPos, sx, sy, Lrange, Orange, sub);

% <lattice> is the image locations to sub-sample from
% calculate <lattice>:
imSize = size(allfilteredPos{1,1});
rowNums = (h+1):subsampleStep:imSize(1)-(h+1);
colNums = (h+1):subsampleStep:imSize(2)-(h+1);
indResponseMatrix = zeros(size(rowNums,2),size(colNums,2),3);
indResponseMatrix(:,:,1) = repmat(rowNums',1,size(colNums,2));
indResponseMatrix(:,:,2) = repmat(colNums,size(rowNums,2),1);
indResponseMatrix(:,:,3) = (indResponseMatrix(:,:,2) - 1) * imSize(1) + indResponseMatrix(:,:,1);
lattice = indResponseMatrix(:,:,3);

% each image is represented by a vector of length <dataDimension>
dataDimension = size(allfilteredPos,2)*size(rowNums,2)*size(colNums,2);

% seletedFeature(i).params is an integer number t, denoting the index
% to precalculated feature responses, t = 1,...,numAllPossibleFeatures
selectedInd = zeros(size(selectedFeatures,1),1); % index of seleted features
for i = 1:size(selectedFeatures,1)
    selectedInd(i)  = selectedFeatures(i).params(1);
end

% calculate responses of selected features <obsPoints>
obsPoints = zeros(nimage,size(selectedFeatures,1));
for indImage = 1:nimage
    tmp = [];
    for indOri = 1:norient
        responseMatrix = allfilteredPos{indImage,indOri}(indResponseMatrix(:,:,3));
        responseVec = reshape(responseMatrix,1,size(responseMatrix,1)*size(responseMatrix,2));
        tmp = [tmp responseVec];
    end
    obsPoints(indImage,:) = tmp(selectedInd);
end

% calculate binary responses (weak classifiers)
thresholdMatrix = repmat(threshold(selectedInd),size(obsPoints,1),1);
obsPointsBinary = obsPoints > thresholdMatrix;
clear obsPoints
clear allfilteredPos

% =====================================================
% Negative images
% =====================================================
I = {};
Iname = negImageNames;
nimage = size(Iname, 1);
Isize = zeros(nimage, 2); 
for i = 1 : nimage
    tmpIm = imread([negFolder '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm), 1); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for i = 1 : nimage
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the testing images to be the same
% filter the neg images
allfilteredNeg = applyfilterfftsame(I, allfilter);  % filter testing images
Cmax(nimage, norient, allfilteredNeg, sx, sy, Lrange, Orange, sub);

negPoints = zeros(nimage,size(selectedFeatures,1));
for indImage = 1:nimage
    tmp = [];
    for indOri = 1:norient
        responseMatrix = allfilteredNeg{indImage,indOri}(indResponseMatrix(:,:,3));
        responseVec = reshape(responseMatrix,1,size(responseMatrix,1)*size(responseMatrix,2));
        tmp = [tmp responseVec];
    end
    negPoints(indImage,:) = tmp(selectedInd);
end

filterBank = struct('name','gabor','paramRange',[]);
filterBank.paramRange = (1:dataDimension)';

thresholdMatrix = repmat(threshold(selectedInd),size(negPoints,1),1);
negPointsBinary = negPoints > thresholdMatrix;

clear allfilteredNeg
