function testModel2(lambdas,posPoints,negPoints,folderName)
% =================================================================
% Test the learned model.
%
% Code was originally designed for comparison adaboost and other 
%   models in testing, thus not in the most compact form. 
% To make it more clear, unrelated code is deleted.
% 
% <lambdas>: weights of selected weak classifiers.
% <posPoints>: a matrix where each row denotes binary responses
%   of a positive example to features in the feature bank.
% <negPoints>: similar to <obsPoints), while each row is binary
%   responses of a negative example.
% <folderName>: output folder of results.
%
% Adaboost implementation follows the version formulated by (Viola, Jones 2001).
% Cascading is NOT performed.
%
% =================================================================

folder = [folderName];
if(~exist(folder,'dir'))
    mkdir(folder);
end

% Ground truth
trueLabel = [ones(size(posPoints,1),1);zeros(size(negPoints,1),1)];
% binary responses of pos and neg images
jointPoints = [posPoints;negPoints];
% strong classifier scores of the images
scoreVec = zeros(size(jointPoints,1),1);
% ROC area
auc = zeros(size(lambdas,1),1);
% error rates
errorRateStrongClassifier = zeros(size(lambdas,1),1);

% Evaluate strong classifiers with 1,2,3... weak classifiers
for k = 1:size(lambdas,1)
    binaryResponseVec = jointPoints(:,k);
    % calculate strong classifier scores
    scoreVec = scoreVec + binaryResponseVec * lambdas(k);
    % strong classifier threshold
    threshold = sum(lambdas(1:k))/2;
    % calculate error rate
    errorRateStrongClassifier(k) = sum((scoreVec>=threshold) ~= trueLabel)/size(jointPoints,1);
    gt = trueLabel; gt(gt==0) = -1;
    % calculate AUC (area under curve) of ROC
    auc(k) = calc_auc(scoreVec,gt);
    disp(['step ' num2str(k) ': strong classifier error=' num2str(errorRateStrongClassifier(k)) ' auc=' num2str(auc(k))]);
end

% display (save) errorRate plot, ROC plot
h=figure;plot(errorRateStrongClassifier);
saveas(h,[folderName '/testerr.jpg']);
xlabel('# selected basis');
title('test error');
close;
h=figure;plot(auc);
saveas(h,[folderName '/testauc.jpg']);
xlabel('# selected basis');
title('test auc');
close
[tp, fp] = roc(gt,scoreVec);
h=figure;getROCfig(fp,tp,'-',sprintf('AUC = %.3f',auc(end)));
saveas(h,[folderName '/testROC.eps']);

save([folderName '/testError.mat'],'auc','errorRateStrongClassifier','posPoints','negPoints','scoreVec','gt','tp','fp');


